function mq_codec_init_tables

%     This file is part of mq_codec.
% 
%     mq_codec is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     mq_codec is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with mq_codec.  If not, see <http://www.gnu.org/licenses/>.
%
% MQ codec_init_tables routine
% 
% Syntax
%     mq_codec_init_tables
% 
% [1] David S. Taubman and Michael W. Marcellin, JPEG2000, Image Compression 
%     Fundamentals, Standards and Practice, Kluwer Academic Publishers, 2002
%
% see also mq_encoder, mq_decoder
%
% Xavier Alameda Pineda (xavi.alameda at gmail.com) v1.0 - 20/May/2009
% Image and Video Processing Group, Technical University of Catalonia

% Definition of the Static Table
% First column ~ sigma_mps
% Second column ~ sigma_lps
% Third column ~ X_s
% Fourth column ~ probaility (hexadecimal)

Static_Table = zeros(47,4,'uint16');

% First column
% Extracted from kakadu software [1]
Static_Table(:,1) = [1, 2, 3, 4, 5,38, 7, 8, 9,10,11,12,13,29,15,16,17,...
                    18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,...
                    35,36,37,38,39,40,41,42,43,44,45,45,46];
% Adjustement from C++ indexes to MATLAB indexes
Static_Table(:,1) = Static_Table(:,1) + 1;

% Second column
% Extracted from kakadu software [1]
Static_Table(:,2) = [1, 6, 9,12,29,33, 6,14,14,14,17,18,20,21,14,14,15,...
                    16,17,18,19,19,20,21,22,23,24,25,26,27,28,29,30,31,...
                    32,33,34,35,36,37,38,39,40,41,42,43,46];
% Adjustement from C++ indexes to MATLAB indexes
Static_Table(:,2) = Static_Table(:,2) + 1;

% Third column
Static_Table(:,3) = zeros(47,1,'uint16');
Static_Table([1 7 15],3) = 1;

% Fourth column
% Extracted from kakadu software [1]
Static_Table(:,4) = uint16(hex2dec(['5601';'3401';'1801';'0AC1';'0521';'0221';'5601';...
                             '5401';'4801';'3801';'3001';'2401';'1C01';'1601';...
                             '5601';'5401';'5101';'4801';'3801';'3401';'3001';...
                             '2801';'2401';'2201';'1C01';'1801';'1601';'1401';...
                             '1201';'1101';'0AC1';'09C1';'08A1';'0521';'0441';...
                             '02A1';'0221';'0141';'0111';'0085';'0049';'0025';...
                             '0015';'0009';'0005';'0001';'5601']));
save('static_table_mq_codec','Static_Table');
