function reg_dec = mq_decoder_init(filename)

%     This file is part of mq_codec.
% 
%     mq_codec is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     mq_codec is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with mq_codec.  If not, see <http://www.gnu.org/licenses/>.
%
% MQ decoder_init routine
% 
% Syntax
%     reg_dec = mq_decoder_init(filename)
%     
% Inputs
%     filename ~ byte-stream's filename
%     
% Outputs
%     reg_dec ~ state-registers for the coder (initialized)
%     
% Note
%     Implemented from p. 482 of [1]
% 
% [1] David S. Taubman and Michael W. Marcellin, JPEG2000, Image Compression 
%     Fundamentals, Standards and Practice, Kluwer Academic Publishers, 2002
%
% see also mq_decoder, mq_fill_lsbs, mq_codec_init_tables
%
% Xavier Alameda Pineda (xavi.alameda at gmail.com) v1.0 - 20/May/2009
% Image and Video Processing Group, Technical University of Catalonia

% % reg_dec Initialitzation
% % ------------------------

% reg_dec values

reg_dec.A = uint16(0);
reg_dec.t =  uint8(0);
reg_dec.C = uint32(0);      % Lower-bound interval
reg_dec.T =  uint8(0);      % Temporary byte reg_decister
reg_dec.L =  int32(0);      % Byte

% Byte-stream init

Temp = load(filename);
if ~isfield(Temp,'byte_stream');
    error('Bad filename.');
end
reg_dec.byte_stream = Temp.byte_stream;
clear Temp

% Decoder init procedure

reg_dec = mq_fill_lsbs(reg_dec);
reg_dec.C = bitshift(reg_dec.C,reg_dec.t);
reg_dec = mq_fill_lsbs(reg_dec);
reg_dec.C = bitshift(reg_dec.C,7);
reg_dec.t = reg_dec.t-7;
reg_dec.A = uint16(hex2dec('8000'));