function reg_enc = mq_encoder_end(reg_enc,filename)

%     This file is part of mq_codec.
% 
%     mq_codec is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     mq_codec is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with mq_codec.  If not, see <http://www.gnu.org/licenses/>.
%
% MQ encoder_end routine
% 
% Syntax
%     reg_enc = mq_encoder_end(reg_enc,filename)
%     
% Inputs
%     reg_enc ~ state-registers for the coder (before termination)
%     filename ~ filename for the byte_stream
%     
% Outputs
%     reg_enc ~ state-registers for the coder (after termination)
%     
% Note
%     Implemented from p. 496 of [1]
% 
% [1] David S. Taubman and Michael W. Marcellin, JPEG2000, Image Compression 
%     Fundamentals, Standards and Practice, Kluwer Academic Publishers, 2002
%
% see also mq_encoder_init, mq_encoder, mq_decoder_init
%
% Xavier Alameda Pineda (xavi.alameda at gmail.com) v1.0 - 20/May/2009
% Image and Video Processing Group, Technical University of Catalonia

% The number of bits we need to flush out of C
nbits = 27-15-reg_enc.t;
% Move the next 8 available bits into the partial byte
reg_enc.C = reg_enc.C * uint32(2^reg_enc.t);
while nbits > 0
    reg_enc = mq_transfer_byte(reg_enc);
    % New value of reg.t is the number of bits just transferred
    nbits = nbits - reg_enc.t;
    % Move bits into available position for next transfer
    reg_enc.C = reg_enc.C * uint32(2^reg_enc.t);
end
reg_enc = mq_transfer_byte(reg_enc);

byte_stream = reg_enc.byte_stream;

save(filename,'byte_stream');