function reg_dec = mq_fill_lsbs(reg_dec)

%     This file is part of mq_codec.
% 
%     mq_codec is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     mq_codec is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with mq_codec.  If not, see <http://www.gnu.org/licenses/>.
%
% MQ fill_lsbs routine
% 
% Syntax
%     reg_dec = mq_fill_lsbs(reg_dec) 
%     
% Inputs
%     reg_dec ~ state-registers for the decoder (before fill)
%     
% Outputs
%     reg_dec ~ state-registers for the coder (after fill)
%     
% Note
%     Implemented from p. 478 of [1]
% 
% [1] David S. Taubman and Michael W. Marcellin, JPEG2000, Image Compression 
%     Fundamentals, Standards and Practice, Kluwer Academic Publishers, 2002
%
% see also mq_decoder_init, mq_decoder, mq_renormalize_once
%
% Xavier Alameda Pineda (xavi.alameda at gmail.com) v1.0 - 20/May/2009
% Image and Video Processing Group, Technical University of Catalonia

reg_dec.t = 8;
if (reg_dec.L == numel(reg_dec.byte_stream)) || ...
        (reg_dec.T == uint8(hex2dec('ff')) && reg_dec.byte_stream(reg_dec.L+1) > hex2dec('8f'))
    % Codeword exhausted; fill C with 1's from now on
    reg_dec.C = reg_dec.C + uint32(hex2dec('ff'));
else
    if reg_dec.T == uint8(hex2dec('ff'))
        reg_dec.t = 7;
    end
    reg_dec.T = reg_dec.byte_stream(reg_dec.L+1);
    reg_dec.L = reg_dec.L+1;
    reg_dec.C = reg_dec.C + bitshift(uint32(reg_dec.T),8-reg_dec.t);
end