function reg_enc = mq_transfer_byte(reg_enc)

%     This file is part of mq_codec.
% 
%     mq_codec is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     mq_codec is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with mq_codec.  If not, see <http://www.gnu.org/licenses/>.
%
% MQ transfer_byte routine
% 
% Syntax
%     reg_enc = mq_transfer_byte(reg_enc)
%     
% Inputs
%     reg_enc ~ state-registers for the coder
%     
% Outputs
%     reg_enc ~ state-registers for the coder
%     
% Note
%     Implemented from p. 479 of [1]
% 
% [1] David S. Taubman and Michael W. Marcellin, JPEG2000, Image Compression 
%     Fundamentals, Standards and Practice, Kluwer Academic Publishers, 2002
%
% see also mq_encoder, mq_put_byte
%
% Xavier Alameda Pineda (xavi.alameda at gmail.com) v1.0 - 20/May/2009
% Image and Video Processing Group, Technical University of Catalonia

% C subsets indexes

C_partial = 20:27;
C_msbs    = 21:28;
C_carry   =    28;

% C Subsets Masks

C_partial_mask = uint32(sum(2.^(C_partial-1)));
C_msbs_mask    =    uint32(sum(2.^(C_msbs-1)));

% ------------------start
% Transfer-byte procedure

% can't propagate any carry past T; need bit stuff
if reg_enc.T == hex2dec('ff'),
%    fprintf('MQ_ENC: Bit stuffing policy. (1)\n');
    reg_enc = mq_put_byte(reg_enc);
    reg_enc.T = uint8(bitshift(bitand(reg_enc.C,C_msbs_mask),-min(C_msbs)+1));
% %     if bitshift(bitand(reg_enc.C,C_msbs_mask),-min(C_msbs)+1) - double(reg_enc.T) ~= 0
% %         error('Cagada');
% %     end
    reg_enc.C = bitand(reg_enc.C,bitcmp(C_msbs_mask));
    reg_enc.t = 7; % transfer 7 bits plus carry
else
    % propagate any carry bit from C into T
    reg_enc.T = reg_enc.T + uint8(bitget(reg_enc.C,C_carry));
    reg_enc.C = bitset(reg_enc.C,C_carry,0);
    reg_enc = mq_put_byte(reg_enc);
    if reg_enc.T == hex2dec('ff'),
%        fprintf('MQ_ENC: Bit stuffing policy. (2)\n');        
        reg_enc.T = uint8(bitshift(bitand(reg_enc.C,C_msbs_mask),-min(C_msbs)+1));
% %         if bitshift(bitand(reg_enc.C,C_msbs_mask),-min(C_msbs)+1) - double(reg_enc.T) ~= 0
% %             error('Cagada');
% %         end
        reg_enc.C = bitand(reg_enc.C,bitcmp(C_msbs_mask));
        reg_enc.t = 7; % transfer 7 bits plus carry
    else
        reg_enc.T = uint8(bitshift(bitand(reg_enc.C,C_partial_mask),-min(C_partial)+1));
% %         if bitshift(bitand(reg_enc.C,C_partial_mask),-min(C_partial)+1) - double(reg_enc.T) ~= 0
% %             error('Cagada');
% %         end
        reg_enc.C = bitand(reg_enc.C,bitcmp(C_partial_mask));
        reg_enc.t = 8; % transfer full byte
    end
end

% Transfer-byte procedure
% --------------------end